package test;

import java.applet.*;
import java.awt.*;

public class Main extends Applet implements Runnable {
	Bar bar=new Bar(300/2-30,370); //反射板を初期座標とともに生成
	Ball ball=new Ball(300/2-4,360); //ボールを初期座標とともに生成
	Block[][] blocks=new Block[10][10]; //ブロックの配列
	Thread gameThread;		// ゲームスレッド
	Image offImage;			// 仮想画面
	int scene;				// シーン（0:タイトル 1:メイン 2:ゲームオーバー）
	int blockDeaths;

	// 初期化
	public void init() {
	    setSize(300, 400);//画面のサイズを指定(横, 縦)
	    ready();//開始準備呼び出し
		setBackground(Color.black);// 背景色を黒色に設定
		setForeground(Color.white);// 前景色を白色に設定
		offImage = createImage(300, 400);// 仮想画面の生成
		addKeyListener(new Key());// キー受け付けオブジェクト生成
		requestFocusInWindow();// フォーカスを要求
		scene = 0;// シーンをタイトルに
	}

	//開始準備
	void ready(){
		// ブロックの初期化
		for(int i=0;i<blocks.length;i++){
			for(int j=0;j<blocks[0].length;j++){
				blocks[i][j]=new Block(j*30,(i+1)*12,10,28,0);
			}
		}
		//反射板を初期座標へ
		bar.x=300/2-30;
		bar.y=370;

		//ボールの初期化
		ball.x=300/2-4;
		ball.y=360;
		ball.dx=2;
		ball.dy =2;
		ball.dead=0;

		blockDeaths=0;
	}

	// ゲームスレッドの開始
	public void start() {
		if(gameThread == null) {
			gameThread = new Thread(this);
			gameThread.start();
		}
	}
	// ゲームスレッドの停止
	public void stop() {
		gameThread = null;
	}

	// ゲームスレッドのメイン
	public void run() {
		while (gameThread == Thread.currentThread()) {
			// ゲームメイン処理
			switch (scene) {
				case 0: gameTitle(); break;
				case 1: gameMain(); break;
				case 2: gameOver(); break;
			}
			// 描画間隔（ミリ秒単位）
			try {
				Thread.sleep(9);
			} catch (InterruptedException e) {
				break;
			}
		}
	}

	// ゲームタイトル処理
	void gameTitle() {
		// エンターキーが押されたらシーンをゲームメインへ
		if (Key.enter) scene = 1;
		Graphics gv = offImage.getGraphics();
		// 裏画面の消去
		gv.clearRect(0, 0, 300, 400);
		// タイトル文字列描画
		gv.setFont(new Font("Arial", Font.BOLD, 35));
		gv.drawString("Block break!", 50, 180);
		gv.setFont(new Font("Arial", Font.PLAIN, 20));
		gv.drawString("HIT ENTER KEY.", 70, 350);
		// 再描画
		repaint();
	}

	// ゲームメイン処理
	void gameMain() {
		//物の移動をそれぞれのクラスから呼び出し
		bar.barMove();
		ball.ballMove();

		//当たり判定メソッド呼び出し
		CheckBarAndBall(bar,ball);
		CheckBlockAndBall(blocks,ball);

		Graphics gv = offImage.getGraphics();// 裏画面の消去

		gv.clearRect(0, 0, 300, 400);
		gv.drawRect(bar.x, bar.y, bar.width, bar.height);// 反射板の描画
		gv.drawOval(ball.x, ball.y, ball.width, ball.height);// ボールの描画
		//ブロックの描画
		for(int i=0;i<blocks.length;i++){
			for(int j=0;j<blocks[0].length;j++){
				if(blocks[i][j].dead==0){
					gv.drawRect(blocks[i][j].x, blocks[i][j].y, blocks[i][j].width, blocks[i][j].height);
				}
			}
		}

		repaint();// 再描画

		if (ball.dead==1 || blockDeaths==blocks.length*blocks[0].length) scene = 2;//ボールが死んだらゲームオーバー画面へ
	}

	// ゲームオーバー処理
	void gameOver() {
		Graphics gv = offImage.getGraphics();
		gv.clearRect(0, 0, 300, 400);// 裏画面の消去

		// ゲームオーバー文字列描画
		gv.setFont(new Font("Arial", Font.BOLD, 28));
		gv.drawString("GAME OVER", 64, 180);
		gv.setFont(new Font("Arial", Font.PLAIN, 20));
		gv.drawString("Push space to return", 55, 250);

		repaint();// 再描画

		//再挑戦
		if (Key.space) {
			ready();// ゲーム準備処理を行い
			scene = 0;// タイトル画面へ移行
		}
	}

	// 描画
	public void paint(Graphics g) {
		requestFocusInWindow();
		// 裏画面から表画面へ転送
		g.drawImage(offImage, 0, 0, this);
	}

	// 更新
	public void update(Graphics g) {
		paint(g);
	}

//////////当たり判定//////////////
	void CheckBarAndBall(Bar bar,Ball ball){
		// 反射板とボールとの当たり判定
		if(bar.x-ball.width/2 <= ball.x && bar.x+bar.width-ball.width/2 >= ball.x
				&& bar.y-ball.height<=ball.y && bar.y+bar.height>=ball.y){
			if((bar.x-ball.width/2 <= ball.x && bar.x-ball.width/2+bar.width/4 >= ball.x)
					||(bar.x+bar.width-ball.width/2 >= ball.x
							&& bar.x+bar.width-ball.width/2-bar.width/4 <= ball.x)){

				if(ball.dx<0){
					ball.dx=-2;
				}else if(ball.dx>0){
					ball.dx=2;
				}

				ball.dy = -1;
			}else
			ball.dy=-2;
		}
	}
	void CheckBlockAndBall(Block[][] blocks,Ball ball){
		//ブロックとボールの当たり判定
		for(int i=0;i<blocks.length;i++){
			for(int j=0;j<blocks[0].length;j++){
				if(blocks[i][j].dead==0){
					if ((blocks[i][j].x-ball.width/2 <= ball.x
							&& blocks[i][j].x+blocks[i][j].width-ball.width/2 >= ball.x)
							&& (blocks[i][j].y+ball.height == ball.y
									|| blocks[i][j].y+blocks[i][j].height == ball.y)) {
							ball.dy=-ball.dy;
							blocks[i][j].dead=1;
							blockDeaths++;
						}
					if ((blocks[i][j].y-ball.height/2 <= ball.y
							&& blocks[i][j].y+blocks[i][j].height-ball.height/2 >= ball.y)
							&& (blocks[i][j].x-ball.width == ball.x
									|| blocks[i][j].x+blocks[i][j].width == ball.x)) {
							ball.dx=-ball.dx;
							blocks[i][j].dead=1;
							blockDeaths++;
						}
				}
			}
		}
	}
}
